<?php

namespace FiloBlu\OrderFlowRectifier\Model\Order\Email\Sender;

use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\DataObject;
use Magento\Framework\Event\ManagerInterface;
use Magento\Payment\Helper\Data as PaymentHelper;
use Magento\Sales\Model\Order;
use Magento\Sales\Model\Order\Address\Renderer;
use Magento\Sales\Model\Order\Email\Container\IdentityInterface;
use Magento\Sales\Model\Order\Email\Container\OrderIdentity;
use Magento\Sales\Model\Order\Email\Container\Template;
use Magento\Sales\Model\ResourceModel\Order as OrderResource;
use Magento\Store\Model\ScopeInterface;

class OrderSender extends \Magento\Sales\Model\Order\Email\Sender\OrderSender
{
    const XML_PATH_OFFLINE_PAYMENT_DETAILS_TEMPLATE = 'order_flow_rectifier/adyen_email_configurations/offline_payment_details_template';
    const ADYEN_EMAIL_CONFIGURATIONS_CONFIG_PATH = 'order_flow_rectifier/adyen_email_configurations/adyen_email_configurations_enabled';

    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var ResourceConnection
     */
    protected $connection;

    /**
     * @param Template $templateContainer
     * @param OrderIdentity $identityContainer
     * @param Order\Email\SenderBuilderFactory $senderBuilderFactory
     * @param \Psr\Log\LoggerInterface $logger
     * @param Renderer $addressRenderer
     * @param PaymentHelper $paymentHelper
     * @param OrderResource $orderResource
     * @param ScopeConfigInterface $globalConfig
     * @param ManagerInterface $eventManager
     * @param ScopeConfigInterface $scopeConfig
     * @param ResourceConnection $connection
     */
    public function __construct(
        Template $templateContainer,
        OrderIdentity $identityContainer,
        \Magento\Sales\Model\Order\Email\SenderBuilderFactory $senderBuilderFactory,
        \Psr\Log\LoggerInterface $logger,
        Renderer $addressRenderer,
        PaymentHelper $paymentHelper,
        OrderResource $orderResource,
        \Magento\Framework\App\Config\ScopeConfigInterface $globalConfig,
        ManagerInterface $eventManager,
        ScopeConfigInterface $scopeConfig,
        ResourceConnection $connection
    ) {
        parent::__construct($templateContainer, $identityContainer, $senderBuilderFactory, $logger, $addressRenderer, $paymentHelper, $orderResource, $globalConfig, $eventManager);
        $this->scopeConfig = $scopeConfig;
        $this->connection = $connection->getConnection();
    }

    /**
     * @param Order $order
     * @param bool $forceSyncMode
     * @return bool
     */
    public function send(Order $order, $forceSyncMode = false)
    {
        $isOrderToPayOffline = $this->isOrderToPayOffline($order);
        if ($isOrderToPayOffline) {

           if (!$order->getOfflinePaymentEmailSent()) {
               $this->identityContainer->setStore($order->getStore());
               $order->setSendEmail($this->identityContainer->isEnabled());
               if ($this->checkAndSend($order)) {
                   if ($this->globalConfig->getValue('sales_email/general/async_sending')) {
                       $order->setEmailSent(null);
                   } else {
                       $order->setEmailSent(false);
                   }
                   $order->setData('offline_payment_email_sent', 1);
                   $this->orderResource->saveAttribute($order, ['email_sent', 'offline_payment_email_sent']);
               }

               $this->orderResource->saveAttribute($order, 'send_email');
           }

           return false;
       } else {
           return parent::send($order, $forceSyncMode);
       }
    }

    /**
     * @param Order $order
     * @return void
     */
    protected function prepareTemplate(Order $order)
    {
        parent::prepareTemplate($order);
        $isOrderToPayOffline = $this->isOrderToPayOffline($order);

        if ($isOrderToPayOffline)
        {
            $payment = $order->getPayment();
            $isPayByLink = false;
            $isBankTransfer = false;
            if ($payment->getMethod() == 'adyen_pay_by_link') {
                $isPayByLink = true;
            } else {
                if ($payment->getCcType() == 'bankTransfer_IBAN') {
                    $isBankTransfer = true;
                }
            }
            $templateVars = $this->templateContainer->getTemplateVars();
            $templateVars['order']['isPayByLink'] = $isPayByLink;
            $templateVars['order']['isBankTransfer'] = $isBankTransfer;
            $this->templateContainer->setTemplateVars($templateVars);

            $templateId = $this->scopeConfig->getValue(
                self::XML_PATH_OFFLINE_PAYMENT_DETAILS_TEMPLATE,
                \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
                $order->getStoreId()
            );
            $this->templateContainer->setTemplateId($templateId);
        }
    }

    /**
     * @param Order $order
     * @return bool
     */
    protected function isOrderToPayOffline(Order $order)
    {
        $isOfflinePaymentMethod = false;
        $paymentRecived = false;

        if (!$this->scopeConfig->isSetFlag(self::ADYEN_EMAIL_CONFIGURATIONS_CONFIG_PATH, ScopeInterface::SCOPE_STORE, $order->getStoreId())) {
            return $isOfflinePaymentMethod;
        }

        $paymentMethod = $order->getPayment()->getMethod();
        $ccType = $order->getPayment()->getCcType();

        if ($paymentMethod == 'adyen_pay_by_link'
            || ($paymentMethod == 'adyen_hpp' && in_array($ccType, ['bankTransfer_IBAN', 'multibanco'])))
        {
            $isOfflinePaymentMethod = true;
            $adyenNotification = $this->connection->getTableName('adyen_notification');
            $query = "SELECT pspreference FROM {$adyenNotification}
                    WHERE merchant_reference = '".$order->getIncrementId()."'
                    AND event_code != 'PENDING'
                    AND success = 'true'";

            $notification = $this->connection->fetchAll($query, [], \Zend_Db::FETCH_OBJ);
            if (count($notification) > 0) {
                $paymentRecived = true;
            }
        }

        return $isOfflinePaymentMethod && !$paymentRecived;
    }
}
